/* DowBld.cmd  Build DOW files for D:\GCP\DATA from D:\GCP\DOW

    $TLIB$: $ &(#) %n - Ver %v, %f $
    TLIB: $ &(#) DOWBLD.CMD - Ver 3, 03-Apr-96,19:20:26 $

    Revisions	09 Feb 96 SHL - Release
		19 Mar 96 SHL - Add by DOW in DowDir logic
                03 Apr 96 SHL - Add embedded date logic

    Build DOW if catalog older than age days in DOW

*/

signal on ERROR
signal on FAILURE name Error
signal on HALT
signal on NOVALUE name Error
signal on SYNTAX name Error

/* Point a data directory and DOW save directory */
DataDir = 'd:\gcp\data'
DowDir = 'd:\gcp\dow'

idDateHow = 'E'		/* C = use CAT file date, D = use DOW date, E = use embedded date */

call LoadFuncs

/*  $TLIB$: 'DOWBLD v%v checking' */
say 'DOWBLD v3 checking' dowDir'.'

/* Build file list */

rc = SysFileTree(DowDir'\*.dow', 'DowList', 'FO')

if rc then do
  say 'Aborting: SysFileTree failed for' DowDir'.'
  exit
end

if DowList.0 = 0 then do
  say 'Qutting: no .DOW files found in 'DataDir'.'
  exit
end

/* Scan DOW's in .DOW directory and build as needed */

do iDow = 1 to DowList.0
  call DoDow
end /* do i */

exit

/* end main */

/*=== DoDow: Process thisDow ===*/

DoDow:

thisDow = DowList.iDow

/* say 'Doing' thisDow '('iDow 'of' DowList.0')' */

s = filespec('N', thisDow)
i = lastpos('.', s)
thisForum = substr(s, 1, i - 1)
newDow = DataDir'\'s

if stream(newDow, 'C', 'QUERY EXISTS') \= '' then
  say 'Skipping' thisForum 'because' newDow 'exists.'
else do
  catFile = DataDir'\'thisForum'.cat'
  if stream(catFile, 'C', 'QUERY EXISTS') = '' then
    say 'Skipping' thisForum 'because' catFile 'not found.'
  else do
    /* Read in DOW file */
    signal on NOTREADY name Error
    call stream thisDow, 'C', 'OPEN READ'
    i = 0
    do while lines(thisDow)
      i = i + 1
      theLines.i = linein(thisDow)
    end
    theLines.0 = i
    call stream thisDow, 'C', 'CLOSE'
    select
    when idDateHow == 'D' then do
      if i \= 2 then do
	say 'Aborting: expected 2 lines in' thisDow'.'
	exit
      end
      sz = stream(thisDow, 'C', 'QUERY DATETIME')
      parse value sz with catDate catTime
      catDate = translate(catDate, '/', '-')
    end
    when idDateHow == 'C' then do
      if i \= 2 then do
	say 'Aborting: expected 2 lines in' thisDow'.'
	exit
      end
      sz = stream(catFile, 'C', 'QUERY DATETIME')
      parse value sz with catDate catTime
      catDate = translate(catDate, '/', '-')
    end
    when idDateHow == 'E' then do
      if i = 2 then do
	/* Must be first time, use DOW date */
	sz = stream(thisDow, 'C', 'QUERY DATETIME')
	parse value sz with catDate catTime
	catDate = translate(catDate, '/', '-')
      end
      else if i = 3 then do
        drop catTime sz
	parse value theLines.3 with 'DOWBLD:' catDate catTime sz
        signal off NOVALUE
	if catTime == '' | sz != '' then do
	  say 'Aborting: expected Last date and time at line 3 of' thisDow'.'
	  exit
	end
        signal on NOVALUE name Error
      end
      else do
	say 'Aborting: expected 2 or 3 lines in' thisDow'.'
	exit
      end
    end
    otherwise do
      say 'Aborting: idDateHow' idDateHow 'unknown.'
      exit
    end
    end /* select */
    catAge = RexxDate_Diff(,catDate)
    if substr(theLines.1, 1, 2) \= 'XA' then do
      say 'Aborting:' thisDow 'contains unexpected line 1:' theLines.1'.'
      exit
    end
    ageDays = ''
    theLibs = ''
    parse value theLines.2 with 'DIR LONG DES AGE:' ageDays 'LIB:'theLibs
    if \ DataType(ageDays, 'W') then do
      say 'Aborting:' thisDow 'contains unexpected line 2:' theLines.2'.'
      exit
    end
    if catAge - ageDays < 0 then do
      c = ageDays - catAge
      select
      when idDateHow == 'C' then
	say 'Will build' thisForum 'in' c 'days. ' catFile 'is' catAge 'days old.'
      when idDateHow == 'D' then
	say 'Will build' thisForum 'in' c 'days.' thisDow 'is' catAge 'days old.'
      when idDateHow == 'E' then
	say 'Will build' thisForum 'in' c 'days. Last scan was' catAge 'days ago on' catDate'.'
      otherwise:
        signal failure
      end
    end
    else do
      c = catAge + 1		    /* Overlap to avoid missing files */
      theLines.2 = 'DIR LONG DES AGE:'c 'LIB:'theLibs
      /* Write new DOW file */
      call stream newDow, 'C', 'OPEN WRITE'
      do i = 1 to 2
	call lineout newDow, theLines.i
      end
      call stream newDow, 'C', 'CLOSE'
      select
      when idDateHow == 'C' then
        nop
      when idDateHow == 'D' then do
        /* DOW date controls selection, touch it to change date */
	call stream thisDow, 'C', 'OPEN WRITE'
	call stream thisDow, 'C', 'SEEK =1'
	call lineout thisDow, theLines.1		/* fixme to touch smarter */
	call stream thisDow, 'C', 'CLOSE'
      end
      when idDateHow == 'E' then do
        /* DOW datestamp on line 3 controls selection change date */
	call stream thisDow, 'C', 'OPEN WRITE'
	if theLines.0 = 2 then
	  call stream thisDow, 'C', 'SEEK <0'
	else do
	  l = length(theLines.1) + length(theLines.2) + 2
	  call stream thisDow, 'C', 'SEEK ='l
        end
	call lineout thisDow, 'DOWBLD:' date('u') time('n')     /* Remember when */
	call stream thisDow, 'C', 'CLOSE'
      end
      otherwise
	signal failure
      end  /* Select */
      say '* Built' newDow 'because update requested every' ageDays 'days.'
      say '  'theLines.2
    end  /* Do */
    signal off NOTREADY
  end
end

return

/* end DoDow */

/*=== LoadFuncs: Load fuctions ===*/

LoadFuncs:

/* Add all Rexx functions */
if RxFuncQuery('SysLoadFuncs') then do
  call RxFuncAdd 'SysLoadFuncs', 'REXXUTIL', 'SysLoadFuncs'
  if RESULT then do
    say 'Aborting: cannot load SysLoadFuncs.'
    exit
  end
  call SysLoadFuncs
end /* end do */

if RxFuncQuery('RexxDate_Init') then do
  call RxFuncAdd 'RexxDate_Init', 'REXXDATE', 'RexxDate_Init'
  if RESULT then do
    say 'Aborting: cannot load RexxDate_Init.'
    exit
  end
  call RexxDate_Init
end /* end do */

return

/* end LoadFuncs */


/*=== Error: Trap ERROR and FAILURE conditions ===*/

Error:

  say
  say condition('C') 'signaled at' SIGL'.'
  say 'Source = ' sourceline(SIGL)
  call SysSleep 2
  if condition('I') = 'CALL' then do
    say 'Returning...'
    return
  end
  else do
   say 'Aborting.'
   exit
  end

/* end Error */

/*=== Halt: Trap HALT condition ===*/

Halt:

  say
  say condition('C') 'signaled at' SIGL
  say 'Source = ' sourceline(SIGL)
  call SysSleep 2
  if condition('I') = 'CALL' then do
    say 'Returning...'
    return
  end
  else do
   say 'Aborting.'
   exit
  end

/* end Halt */

/* The end */

